var logger = org.slf4j.LoggerFactory.getLogger("funksvd.js");
var lk = org.grouplens.lenskit;
var dataset = properties.get("dataset");

recipe.addEval("Coverage");
recipe.addEval("NDCG");
recipe.addEval("HLUtility");
recipe.addEval("MAE");
recipe.addEval("RMSE");

logger.info("Configuring FunkSVD for {}", dataset);

importPackage(org.grouplens.lenskit);
importPackage(org.grouplens.lenskit.params);
importPackage(org.grouplens.lenskit.norm);
importPackage(org.grouplens.lenskit.baseline);
importPackage(org.grouplens.lenskit.svd);
importPackage(org.grouplens.lenskit.svd.params);

// Set up a trial. This configures a new algorithm with the parameters
// in trial. Trial can safely be mutated after the trial is made to
// configure a new trial.
function makeTrial(trial) {
  rec = recipe.addAlgorithm();
  rec.name = "FunkSVD";
  rec.factory.setComponent(RatingPredictor, FunkSVDRatingPredictor);
  for (var f in trial) {
    var t = trial[f];
    t.apply(rec);
    for (k in t.attrs) {
      rec.attributes.put(k, t.attrs[k]);
    }
  }
}

var ks = [10, 20, 30, 40, 50, 75, 100];
var itercts = [50, 75, 100, 125, 150, 200];
var clamping = ["Identity", "Range"];
var baseline = ["ItemMean", "UserMean", "ItemUserMean"];

function makeK(k) {
  return {
    attrs: { k: k },
    apply: function(rec) {
      rec.factory.set(FeatureCount, k);
    }
  };
}

function makeBaseline(bl, smooth) {
  return {
    attrs: { Baseline: bl, Smoothing: smooth },
    apply: function(rec) {
      var blobj;
      if (bl == "None")
        blobj = org.grouplens.lenskit.baseline.ConstantPredictor;
      else
        blobj = org.grouplens.lenskit.baseline[bl + "Predictor"];
      rec.factory.setComponent(BaselinePredictor, blobj);
      if (smooth)
        rec.factory.set(MeanSmoothing, smooth);
    }
  };
}

function makeIterCount(n) {
  return {
    attrs: { IterationCount: n },
    apply: function(rec) {
      rec.factory.set(IterationCount, n);
    }
  };
}

function makeClamping(clamp) {
  return {
    attrs: { Clamping: clamp },
    apply: function(rec) {
      var f;
      if (clamp == "Range")
        rec.factory.setComponent(ClampingFunction,
                                 org.grouplens.lenskit.util.DoubleFunction,
                                 RatingRangeClamp);
    }
  };
};

function makeLearningRate(rate) {
  return {
    attrs: {LRate: rate},
    apply: function(rec) {
      rec.factory.set(LearningRate, rate);
    }
  };
}

function makeRegularization(reg) {
  return {
    attrs: {Reg: reg},
    apply: function(rec) {
      rec.factory.set(RegularizationTerm, reg);
    }
  };
}

ks.forEach(function(k) {
  var trial = {
    clamping: makeClamping("Range"),
    baseline: makeBaseline("ItemUserMean", 25),
    features: makeK(k),
    count: makeIterCount(100),
    lrate: makeLearningRate(0.001),
    reg: makeRegularization(0.015)
  };

  makeTrial(trial);
  
  if (dataset == "ml-100k") {
    var t = trial.lrate;
    trial.lrate = makeLearningRate(0.002);
    makeTrial(trial);
    trial.lrate = t;
  }

  if (dataset != "ml-10m") {
    trial.clamping = makeClamping("Identity");
    makeTrial(trial);
  }
});
