var logger = org.slf4j.LoggerFactory.getLogger("itemitem.js");
var lk = org.grouplens.lenskit;
var dataset = properties.get("dataset");

recipe.addEval("Coverage");
recipe.addEval("NDCG");
recipe.addEval("HLUtility");
recipe.addEval("MAE");
recipe.addEval("RMSE");

function extend(parent, child) {
  for (k in parent) {
    child[k] = parent[k];
  }
  return child;
}

importPackage(org.grouplens.lenskit);
importPackage(org.grouplens.lenskit.params);
importPackage(org.grouplens.lenskit.norm);
importPackage(org.grouplens.lenskit.baseline);
importPackage(org.grouplens.lenskit.knn);
importPackage(org.grouplens.lenskit.knn.params);
importPackage(org.grouplens.lenskit.knn.item);

var nss = [10, 20, 30, 50, 75, 100];
function NbrSize(n) {
  this.attrs = {nnbrs: n}
  this.nnbrs = n
}
NbrSize.prototype.apply = function(rec) {
  rec.factory.set(NeighborhoodSize, this.nnbrs);
};

var sims = ["Cosine", "Pearson"];
function Sim(sim, adj) {
  this.attrs = {sim: sim};
  this.simDamping = 0;
  if (adj == "shrink") {
    this.attrs.Adj = "Shrink 100";
    this.simDamping = 100;
  } else if (adj == "weight") {
    this.attrs.Adj = "Weight 50";
    this.weightedSim = SignificanceWeight;
    this.weight = 50;
  } else if (!adj) {
    this.attrs.Adj = "None";
  } else {
    logger.error("Invalid adjustment {}", adj);
    throw {message: "Invalid adjustment", reason: adj};
  }

  if (sim == "Cosine") {
    this.sim = lk.knn.CosineSimilarity;
  } else if (sim == "Pearson") {
    this.sim = lk.knn.PearsonCorrelation;
  } else {
    logger.error("Invalid similarity function {}", sim);
    throw {error: "invalid similarity", arg: sim}
  }
}
Sim.prototype.apply = function(rec) {
  rec.factory.setComponent(ItemSimilarity, Similarity, this.sim);
  rec.factory.set(SimilarityDamping, this.simDamping);
  if (this.weightedSim) {
    rec.factory.set(WeightThreshold, this.weight);
    rec.factory.setComponent(WeightedSimilarity, Similarity, this.weightedSim);
  }
}

function findBaseline(bl) {
  if (bl == "None")
    return null;
  else
    return lk.baseline[bl + "Predictor"];
}

var norms = ["None", "GlobalMean", "UserMean",
             "Variance", "ItemMean", "ItemUserMean"];
function Norm(norm) {
  this.attrs = {Norm: norm}
  if (norm == "None") {
    this.norm = IdentityVectorNormalizer
  } else if (norm == "Variance") {
    this.norm = UserVarianceNormalizer;
  } else {
    this.baseline = findBaseline(norm);
    this.norm = BaselineSubtractingNormalizer;
  }
}
Norm.prototype = {
  apply: function(rec) {
    rec.factory.setComponent(VectorNormalizer, this.norm);
    if (this.baseline)
      rec.factory.setComponent(NormalizerBaseline, BaselinePredictor, this.baseline);
  }
};

function Baseline(bl) {
  this.attrs = { Baseline: bl }
  this.baseline = findBaseline(bl);
}
Baseline.prototype = {
  apply: function (rec) {
    rec.factory.setComponent(BaselinePredictor, this.baseline);
  }
};

// Set up a trial. This configures a new algorithm with the parameters
// in trial. Trial can safely be mutated after the trial is made to
// configure a new trial.
function makeTrial(trial) {
  rec = recipe.addAlgorithm();
  rec.name = "ItemItem";
  rec.factory.setComponent(RatingPredictor, ItemItemRatingPredictor);
  // rec.factory.setComponent(ItemRecommender, ItemItemRatingRecommender);
  for (var f in trial) {
    var t = trial[f];
    t.apply(rec);
    for (k in t.attrs) {
      rec.attributes.put(k, t.attrs[k]);
    }
  }
}

// Compute the neighborhood size trials
nss.forEach(function(n) {
  var trial = {
    nnbrs: new NbrSize(n),
    baseline: new Baseline("ItemUserMean")
  };
  [null, "shrink"].forEach(function(adj) {
    makeTrial(extend(trial, {
      similarity: new Sim("Pearson", adj),
      norm: new Norm("None")
    }));
    ["None", "ItemMean", "UserMean", "ItemUserMean"].forEach(function(n) {
      makeTrial(extend(trial, {
        similarity: new Sim("Cosine", adj),
        norm: new Norm(n)
      }));
    });
  });
});
